@@ -319,7 +319,7 @@ class OpsaniDevConfiguration(servo.BaseConfiguration):
 
 class OpsaniDevChecks(servo.BaseChecks):
     config: OpsaniDevConfiguration
-    optimizer: servo.OpsaniOptimizer
+    optimizer: servo.configuration.OptimizerTypes
 
     # FIXME make this a property of worklod helper?
     @property
@@ -359,7 +359,11 @@ class OpsaniDevChecks(servo.BaseChecks):
     @servo.checks.require("Optimizer Configuration")
     def check_optimizer(self) -> None:
         assert isinstance(
-            self.optimizer, servo.OpsaniOptimizer
+            self.optimizer,
+            servo.configuration.OpsaniOptimizer,
+        ) or isinstance(
+            self.optimizer,
+            servo.configuration.AppdynamicsOptimizer,
         ), f"Opsani Dev connector is incompatible with non OpsaniOptimizer type {self.optimizer.__class__.__name__}"
 
     @servo.checks.require("Connectivity to Kubernetes")
@@ -935,7 +939,6 @@ class OpsaniDevChecks(servo.BaseChecks):
 
     @servo.check("Tuning pod is running")
     async def check_tuning_is_running(self) -> None:
-
         if self.config.create_tuning_pod:
             # Generate a KubernetesConfiguration to initialize the optimization class
             kubernetes_config = self.config.generate_kubernetes_config()
